package com.io.yutian.elementoriginlib.command;

import com.io.yutian.elementoriginlib.command.argument.Argument;
import org.bukkit.command.CommandSender;

import java.util.ArrayList;
import java.util.List;

public abstract class ICommand {

    private String name;
    private String description;

    private List<CommandNode> commandNodes = new ArrayList<>();
    private List<Argument> arguments = new ArrayList<>();

    public ICommand(String name) {
        this(name, null);
    }

    public ICommand(String name, String description) {
        this.name = name;
        this.description = description;
    }

    public void executes(CommandContext commandContext) {
    }

    public boolean emptyExecutes(CommandSender commandSender) {
        return false;
    }

    public boolean hasPermission(CommandSender sender) {
        return sender.isOp() || sender.hasPermission(getPermissionPrefix()+"."+name);
    }

    public String getPermissionPrefix() {
        return "command."+name;
    }

    public ICommand addArgument(Argument argument) {
        arguments.add(argument);
        return this;
    }

    public ICommand addCommandNode(CommandNode commandNode) {
        this.commandNodes.add(commandNode);
        return this;
    }

    public List<CommandNode> getCommandNodes() {
        return commandNodes;
    }

    public List<Argument> getArguments() {
        return arguments;
    }

    public String getName() {
        return name;
    }

    public String getDescription() {
        return description;
    }

}
