package com.io.yutian.elementoriginlib.command;

import com.io.yutian.elementoriginlib.command.handler.CommandHandler;
import com.io.yutian.elementoriginlib.command.list.CommandHelp;
import org.bukkit.Bukkit;
import org.bukkit.command.Command;
import org.bukkit.command.CommandMap;
import org.bukkit.plugin.Plugin;
import org.jetbrains.annotations.NotNull;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class SimpleCommandManager implements ICommandManager {

    protected static Map<String, Command> bukkitCommandMap = new HashMap<>();
    protected static CommandMap commandMap;

    private final Plugin plugin;
    private final String name;

    @NotNull
    private List<ICommand> commands;


    public SimpleCommandManager(Plugin plugin, String name) {
        this(plugin, name, new ArrayList<>());
    }

    public SimpleCommandManager(Plugin plugin, String name, @NotNull List<ICommand> commands) {
        this.plugin = plugin;
        this.name = name;
        this.commands = commands;
        register(new CommandHelp(this));
    }

    public void register(@NotNull ICommand command) {
        if (command == null) {
            return;
        }
        commands.add(command);
        if (command instanceof IAlias alias) {
            String[] array = alias.getAlias();
            for (String s : array) {
                registerPluginBukkitCommand(plugin, s, command);
            }
        }
    }

    public void unregisterAll() {
        for (ICommand command : commands) {
            if (command instanceof IAlias) {
                unregister(command);
            }
        }
    }

    public static void unregister(ICommand command) {
        if (!(command instanceof IAlias)) {
            return;
        }
        try {
            Map<String, Command> map = (Map<String, Command>) commandMap.getClass().getMethod("getKnownCommands").invoke(commandMap);
            for (String name : ((IAlias) command).getAlias()) {
                map.remove(name);
                Command bukkitCommand = bukkitCommandMap.get(name);
                bukkitCommand.unregister(commandMap);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void registerPluginCommand(@NotNull Plugin plugin, String commandName) {
        Bukkit.getPluginCommand(commandName).setExecutor(new CommandHandler(plugin, this));
    }

    protected static void registerPluginBukkitCommand(Plugin plugin, String name, ICommand command) {
        SimpleCommandHandler simpleCommandHandler = new SimpleCommandHandler(name, command);
        bukkitCommandMap.put(name, simpleCommandHandler);
        commandMap.register(plugin.getName(), simpleCommandHandler);
    }

    public Plugin getPlugin() {
        return plugin;
    }

    @NotNull
    @Override
    public String getName() {
        return name;
    }

    @NotNull
    @Override
    public List<ICommand> getCommands() {
        return commands;
    }

    static {
        try {
            Class<?> c = Bukkit.getServer().getClass();
            for (Method method : c.getDeclaredMethods()) {
                if (method.getName().equals("getCommandMap")) {
                    commandMap = (CommandMap) method.invoke(Bukkit.getServer(), new Object[0]);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

}
