package com.io.yutian.elementoriginlib.command.argument;

import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

public class ArgumentValue {

    private static final Map<Class<?>, Class<?>> PRIMITIVE_TO_WRAPPER = new HashMap<>();

    static {
        PRIMITIVE_TO_WRAPPER.put(boolean.class, Boolean.class);
        PRIMITIVE_TO_WRAPPER.put(byte.class, Byte.class);
        PRIMITIVE_TO_WRAPPER.put(short.class, Short.class);
        PRIMITIVE_TO_WRAPPER.put(char.class, Character.class);
        PRIMITIVE_TO_WRAPPER.put(int.class, Integer.class);
        PRIMITIVE_TO_WRAPPER.put(long.class, Long.class);
        PRIMITIVE_TO_WRAPPER.put(float.class, Float.class);
        PRIMITIVE_TO_WRAPPER.put(double.class, Double.class);
    }

    private Object value;

    public ArgumentValue(Object value) {
        this.value = value;
    }

    public Object getValue() {
        return value;
    }

    public <V> V get(Class<V> clazz) {
        if (PRIMITIVE_TO_WRAPPER.getOrDefault(clazz, clazz).isAssignableFrom(value.getClass())) {
            return (V) value;
        }
        return null;
    }

    public String getString() {
        return (String) value;
    }

    public int getInt() {
        return (Integer) value;
    }

    public double getDouble() {
        return (Double) value;
    }

    public boolean getBoolean() {
        return (Boolean) value;
    }

    public UUID getUUID() {
        return (UUID) value;
    }

    @Override
    public String toString() {
        return "ArgumentValue{" +
                "value=" + value +
                '}';
    }
}
