package com.io.yutian.elementoriginlib.command.handler;

import com.io.yutian.elementoriginlib.command.*;
import com.io.yutian.elementoriginlib.command.argument.Argument;
import com.io.yutian.elementoriginlib.command.argument.ArgumentValue;
import com.io.yutian.elementoriginlib.lang.Lang;
import org.bukkit.command.Command;
import org.bukkit.command.CommandExecutor;
import org.bukkit.command.CommandSender;
import org.bukkit.command.TabCompleter;
import org.bukkit.plugin.Plugin;

import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

public class CommandHandler implements CommandExecutor, TabCompleter {

    private ICommandManager commandManager;

    public CommandHandler(Plugin plugin, ICommandManager commandManager) {
        this.commandManager = commandManager;
    }

    public void execute(CommandSender sender, String label, String[] args) {
        if (args.length == 0) {
            execute(sender, label, new String[]{"help", "1"});
            return;
        }
        List<ICommand> commands = commandManager.getCommands();
        String command = args[0];
        Stream<ICommand> stream = commands.stream().filter((c) -> c.getName().equalsIgnoreCase(command));
        Optional<ICommand> optional = stream.findFirst();
        if (!optional.isPresent()) {
            sender.sendMessage(Lang.get("command-unknown", command));
            return;
        }
        ICommand iCommand = optional.get();
        if (!iCommand.hasPermission(sender)) {
            sender.sendMessage(Lang.get("command-no-permission"));
            return;
        }
        List<CommandNode> commandNodes = iCommand.getCommandNodes();
        StringBuilder stringBuilder = new StringBuilder();
        for (int i = 0; i < args.length; i++) {
            stringBuilder.append(args[i]);
            if (i < args.length - 1) {
                stringBuilder.append(" ");
            }
        }
        String commandString = stringBuilder.toString();
        if (commandNodes.size() == 0) {
            Map<String, ArgumentValue> map = new HashMap<>();
            if (iCommand.getArguments().size() > 0) {
                int argSize = args.length - 1;
                List<Argument> arguments = iCommand.getArguments();
                int k = 0;
                if (arguments.get(arguments.size()-1).isOptional()) {
                    k++;
                }
                if (argSize < arguments.size()-k) {
                    sender.sendMessage(Lang.get("command-short-arg"));
                    return;
                }
                for (int l = 0; l < arguments.size(); l++) {
                    Argument a = arguments.get(l);
                    int index = l + 1;
                    if (index >= args.length) {
                        break;
                    }
                    String arg = args[index];
                    if (!a.getArgumentsType().test(arg)) {
                        sender.sendMessage(Lang.get("command-unknown-arg", index+1, arg));
                        return;
                    }
                }
                map = parseArgumentValue(sender, arguments, args, 0);
            }
            iCommand.executes(new CommandContext(commandString, label, sender, map));
            return;
        }

        int nodeSize = args.length - 1;

        if (commandNodes.size() > 0 && nodeSize == 0) {
            if (!iCommand.emptyExecutes(sender)) {
                sender.sendMessage(Lang.get("command-short-arg"));
            }
            return;
        }

        String mainNode = args[1];
        Stream<CommandNode> nodeStream = commandNodes.stream().filter((n) -> n.getName().equalsIgnoreCase(mainNode) || n.getAlias().contains(mainNode));
        Optional<CommandNode> nodeOptional = nodeStream.findFirst();
        if (!nodeOptional.isPresent()) {
            sender.sendMessage(Lang.get("command-unknown-arg", 2, mainNode));
            return;
        }
        CommandNode node = nodeOptional.get();

        if (node.getChildrens().size() > 0) {
            checkClidren(commandString, label, sender, 1, args, node);
        } else {
            if (node.getCommand() != null) {
                Map<String, ArgumentValue> map = new HashMap<>();
                if (node.getArguments().size() > 0) {
                    int argSize = args.length - 2;
                    List<Argument> arguments = node.getArguments();
                    int k = 0;
                    if (arguments.get(arguments.size()-1).isOptional()) {
                        k++;
                    }
                    if (argSize < arguments.size()-k) {
                        sender.sendMessage(Lang.get("command-short-arg"));
                        return;
                    }
                    for (int l = 0; l < arguments.size(); l++) {
                        Argument a = arguments.get(l);
                        int index = l + 1;
                        if (index >= args.length) {
                            break;
                        }
                        if (index+1 >= args.length) {
                            break;
                        }
                        String arg = args[index+1];
                        if (!a.getArgumentsType().test(arg)) {
                            sender.sendMessage(Lang.get("command-error-arg", index+1, arg));
                            return;
                        }
                    }
                    map = parseArgumentValue(sender, node.getArguments(), args, 1);
                }
                node.getCommand().run(new CommandContext(commandString, label, sender, map));
            } else {
                sender.sendMessage(Lang.get("command-unknown-arg", 3, mainNode));
            }
        }

    }

    private Map<String, ArgumentValue> parseArgumentValue(CommandSender commandSender, List<Argument> argumentList, String[] args, int i) {
        Map<String, ArgumentValue> map = new HashMap<>();
        List<Argument> arguments = argumentList;
        for (int l = 0; l < arguments.size(); l++) {
            Argument a = arguments.get(l);
            if (i+1+l >= args.length) {
                if (a.isOptional()) {
                    map.put(a.getName(), new ArgumentValue(a.getDefaultValue()));
                }
                return map;
            }
            String arg = args[i+1+l];
            if (!a.getArgumentsType().test(arg)) {
                continue;
            }
            ArgumentValue argumentValue = new ArgumentValue(a.getArgumentsType().get(arg));
            map.put(a.getName(), argumentValue);
        }
        return map;
    }

    private void checkClidren(String commandString, String label, CommandSender sender, int i, String[] args, CommandNode node) {
        i++;
        if (i >= args.length) {
            if (node.getCommand() == null) {
                sender.sendMessage(Lang.get("command-short-arg"));
            } else {
                node.getCommand().run(new CommandContext(commandString, label, sender, new HashMap<>()));
            }
            return;
        }
        String s = args[i];
        Stream<CommandNode> nodeStream = node.getChildrens().stream().filter((n) -> n.getName().equalsIgnoreCase(s) || n.getAlias().contains(s));
        Optional<CommandNode> nodeOptional = nodeStream.findFirst();
        if (!nodeOptional.isPresent()) {
            sender.sendMessage(Lang.get("command-unknown-arg", i+1, s));
            return;
        }
        CommandNode node1 = nodeOptional.get();

        if (node1.getChildrens().size() > 0) {
            checkClidren(commandString, label, sender, i, args, node1);
        } else {
            if (node1.getCommand() != null) {
                Map<String, ArgumentValue> map = new HashMap<>();
                if (node1.getArguments().size() > 0) {
                    int argSize = args.length - i - 1;
                    List<Argument> arguments = node1.getArguments();
                    int k = 0;
                    if (arguments.get(arguments.size()-1).isOptional()) {
                        k++;
                    }
                    if (argSize < arguments.size()-k) {
                        sender.sendMessage(Lang.get("command-short-arg"));
                        return;
                    }
                    for (int l = 0; l < arguments.size(); l++) {
                        Argument a = arguments.get(l);
                        int index = i + l + 1;
                        if (index >= args.length) {
                            break;
                        }
                        String arg = args[index];
                        if (!a.getArgumentsType().test(arg)) {
                            sender.sendMessage(Lang.get("command-unknown-arg", index+1, arg));
                            return;
                        }
                    }
                    map = parseArgumentValue(sender, node1.getArguments(), args, i);
                }
                node1.getCommand().run(new CommandContext(commandString, label, sender, map));
            } else {
                sender.sendMessage(Lang.get("command-unknown-arg", i+1, s));
            }
        }

    }

    @Override
    public boolean onCommand(CommandSender sender, Command command, String label, String[] args) {
        execute(sender, label, args);
        return true;
    }

    @Override
    public List<String> onTabComplete(CommandSender sender, Command command, String alias, String[] args) {
        List<String> list = new ArrayList<>();
        int index = args.length;
        String arg = args[index-1];
        List<ICommand> commands = commandManager.getCommands();
        if (index == 1) {
            List<ICommand> commandList = commands.stream().filter((c)->c.getName().startsWith(arg)).collect(Collectors.toList());
            if (commandList.size() > 0) {
                commandList.forEach(c-> {
                    if (c.hasPermission(sender)) {
                        list.add(c.getName());
                    }
                });
                return list;
            }
            commandList = commands.stream().filter((c)->c.getName().contains(arg)).collect(Collectors.toList());
            if (commandList.size() > 0) {
                commandList.forEach(c-> {
                    if (c.hasPermission(sender)) {
                        list.add(c.getName());
                    }
                });
                return list;
            }
        } else {
            Optional<ICommand> iCommandOptional = commands.stream().filter((c)->c.getName().equalsIgnoreCase(args[0])).findFirst();
            if (!iCommandOptional.isPresent()) {
                return list;
            }
            ICommand iCommand = iCommandOptional.get();
            if (!iCommand.hasPermission(sender)) {
                return list;
            }
            if (iCommand instanceof ITabCompleter) {
                ITabCompleter tabCompleter = (ITabCompleter) iCommand;
                return tabCompleter.onTabComplete(sender, args, index-2, arg);
            } else {
                Map<Integer, List<String>> map = new HashMap<>();
                if (iCommand.getCommandNodes().size() > 0) {
                    List<String> list1 = new ArrayList<>();
                    for (CommandNode node : iCommand.getCommandNodes()) {
                        list1.add(node.getName());
                        list1.addAll(node.getAlias());
                        if (index >= 2) {
                            if (!node.getName().equalsIgnoreCase(args[1])) {
                                continue;
                            }
                        }
                        if (node.getChildrens().size() > 0) {
                            getTabComplete(node, 2, map);
                        } else if (node.getArguments().size() > 0) {
                            List<Argument> arguments = node.getArguments();
                            for (int l = 0; l < arguments.size(); l++) {
                                Argument argument = arguments.get(l);
                                if (argument.getSuggest() != null) {
                                    map.put(2+l+1, argument.getSuggest().getSuggest());
                                    continue;
                                }
                                map.put(2+l+1, Arrays.asList("<"+argument.getName()+">"));
                            }
                        }
                    }
                    map.put(2, list1);
                    return preseSuggest(map.getOrDefault(index, list), arg);
                } else if (iCommand.getArguments().size() > 0) {
                    List<Argument> arguments = iCommand.getArguments();
                    for (int l = 0; l < arguments.size(); l++) {
                        Argument argument = arguments.get(l);
                        if (argument.getSuggest() != null) {
                            map.put(1+l+1, argument.getSuggest().getSuggest());
                            continue;
                        }
                        map.put(1+l+1, Arrays.asList("<"+argument.getName()+">"));
                    }
                    return preseSuggest(map.getOrDefault(index, list), arg);
                }
            }
        }
        return preseSuggest(list, arg);
    }

    public static List<String> preseSuggest(List<String> list, String arg) {
        List<String> newList = new ArrayList<>();
        List<String> list1 = list.stream().filter((c)->c.startsWith(arg)||c.toLowerCase().startsWith(arg.toLowerCase())).collect(Collectors.toList());
        List<String> list2 = list.stream().filter((c)->c.contains(arg)||c.toLowerCase().contains(arg.toLowerCase())).collect(Collectors.toList());
        List<String> list3 = list.stream().filter((c)->c.equalsIgnoreCase(arg)|| c.equalsIgnoreCase(arg)).collect(Collectors.toList());
        newList.addAll(list1);
        newList.addAll(list2);
        newList.addAll(list3);
        return newList;
    }

    private void getTabComplete(CommandNode node, int i, Map<Integer, List<String>> map) {
        i++;
        List<String> list = map.getOrDefault(i, new ArrayList<>());
        for (CommandNode c : node.getChildrens()) {
            list.add(c.getName());
            if (c.getChildrens().size() > 0) {
                getTabComplete(c, i, map);
            } else if (c.getArguments().size() > 0) {
                List<Argument> arguments = c.getArguments();
                for (int l = 0; l < arguments.size(); l++) {
                    Argument argument = arguments.get(l);
                    if (argument.getSuggest() != null) {
                        map.put(i+l+1, argument.getSuggest().getSuggest());
                        continue;
                    }
                    map.put(i+l+1, Arrays.asList("<"+argument.getName()+">"));
                }
            }
        }
        map.put(i, list);
    }

}
