package com.io.yutian.elementoriginlib.gui;

import com.io.yutian.elementoriginlib.gui.button.Button;
import com.io.yutian.elementoriginlib.gui.button.ButtonHandler;
import com.io.yutian.elementoriginlib.gui.button.ItemButton;
import com.io.yutian.elementoriginlib.gui.button.ItemSlotButton;
import net.kyori.adventure.text.Component;
import org.bukkit.Bukkit;
import org.bukkit.Material;
import org.bukkit.entity.Player;
import org.bukkit.event.inventory.InventoryClickEvent;
import org.bukkit.event.inventory.InventoryCloseEvent;
import org.bukkit.inventory.Inventory;
import org.bukkit.inventory.InventoryHolder;
import org.bukkit.inventory.ItemStack;
import org.jetbrains.annotations.NotNull;

public abstract class IGui implements InventoryHolder {

    public Inventory inventory;
    public Player player;

    private Component title;
    private int size;

    public IGui(Player player, Component title, int size) {
        this.inventory = Bukkit.createInventory(this, size, title);
        this.title = title;
        this.size = size;
        this.player = player;
    }

    public abstract void init();

    public abstract void handler(Player player, int slot, InventoryClickEvent event);

    public void close(InventoryCloseEvent event) {
    }

    public void open() {
        player.openInventory(inventory);
    }

    public void clickButton(InventoryClickEvent event, int slot, Button button) {
        if (button instanceof ItemButton itemButton) {
            ItemStack itemStack = event.getCurrentItem();
            ItemStack item = event.getCursor();
            if (itemButton.isItem(itemStack)) {
                if (item != null & !item.getType().equals(Material.AIR)) {
                    if (itemButton.getPutPredicate() != null && !itemButton.getPutPredicate().test(player, item)) {
                        return;
                    }
                    if (itemButton.getClickItemConsumer() != null) {
                        itemButton.getClickItemConsumer().accept(player, item);
                    }
                    player.setItemOnCursor(null);
                    if (itemButton.getPutItemFunction() != null) {
                        ItemStack itemStack1 = itemButton.getPutItemFunction().apply(item);
                        itemButton.setItem(itemStack1);
                        inventory.setItem(slot, itemStack1);
                    } else {
                        itemButton.setItem(item);
                        inventory.setItem(slot, item);
                    }
                }
            } else {
                if (item == null || item.getType().equals(Material.AIR)) {
                    if (itemButton.getClickItemConsumer() != null) {
                        itemButton.getClickItemConsumer().accept(player, item);
                    }
                    player.setItemOnCursor(itemStack);
                    itemButton.setItem(null);
                    inventory.setItem(slot, button.getItemStack());
                } else {
                    if (itemButton.getPutPredicate() != null && !itemButton.getPutPredicate().test(player, item)) {
                        return;
                    }
                    player.setItemOnCursor(itemStack);
                    if (itemButton.getPutItemFunction() != null) {
                        ItemStack itemStack1 = itemButton.getPutItemFunction().apply(item);
                        if (itemButton.getClickItemConsumer() != null) {
                            itemButton.getClickItemConsumer().accept(player, itemStack1);
                        }
                        itemButton.setItem(itemStack1);
                        inventory.setItem(slot, itemStack1);
                    } else {
                        if (itemButton.getClickItemConsumer() != null) {
                            itemButton.getClickItemConsumer().accept(player, item);
                        }
                        itemButton.setItem(item);
                        inventory.setItem(slot, item);
                    }
                }
            }
        } else if (button instanceof ItemSlotButton itemSlotButton) {
            ItemStack itemStack = event.getCurrentItem();
            ItemStack item = event.getCursor();
            if (item == null || item.getType().equals(Material.AIR)) {
                if (itemStack == null || itemStack.getType().equals(Material.AIR)) {
                    return;
                }
                inventory.setItem(slot, null);
                itemSlotButton.setItemStack(null);
                player.setItemOnCursor(itemStack);
                if (itemSlotButton.getPickItemFunction() != null) {
                    itemSlotButton.getPickItemFunction().accept(itemStack);
                }
            } else {
                if (itemStack == null || itemStack.getType().equals(Material.AIR)) {
                    if (!itemSlotButton.isCanPut()) {
                        return;
                    }
                    player.setItemOnCursor(null);
                    itemSlotButton.setItemStack(item);
                    inventory.setItem(slot, item);
                    if (itemSlotButton.getPutItemFunction() != null) {
                        itemSlotButton.getPutItemFunction().accept(item);
                    }
                } else {
                    if (!itemSlotButton.isCanPut()) {
                        return;
                    }
                    player.setItemOnCursor(itemStack);
                    itemSlotButton.setItemStack(item);
                    inventory.setItem(slot, item);
                    if (itemSlotButton.getPutItemFunction() != null) {
                        itemSlotButton.getPutItemFunction().accept(item);
                    }
                }
            }
        } else if (button instanceof ButtonHandler buttonHandler) {
            buttonHandler.handler(event, slot, button);
        }
    }

    @NotNull
    @Override
    public Inventory getInventory() {
        return inventory;
    }

}
