package com.io.yutian.elementoriginlib.gui;

import com.io.yutian.elementoriginlib.ElementOriginLib;
import com.io.yutian.elementoriginlib.gui.button.Button;
import com.io.yutian.elementoriginlib.gui.button.ClickType;
import com.io.yutian.elementoriginlib.gui.button.ItemButton;
import net.kyori.adventure.text.Component;
import org.bukkit.Sound;
import org.bukkit.entity.Player;
import org.bukkit.event.inventory.InventoryAction;
import org.bukkit.event.inventory.InventoryClickEvent;

import java.util.HashMap;
import java.util.Map;

public class PageGui extends IGui {

    private int page = 1;
    private int maxPage;

    private Map<Integer, Page> pages = new HashMap<>();

    public PageGui(Player player, Component title, int size, int maxPage) {
        super(player, title, size);
        this.maxPage = maxPage;
    }

    public void initButton() {
        initButton(this.page);
    }

    public void initButton(int page) {
        if (page > pages.size()) {
            return;
        }
        for (int i = 0; i < inventory.getSize(); i++) {
            inventory.setItem(i, null);
        }
        if (pages.containsKey(page)) {
            Page page1 = pages.get(page);
            for (Map.Entry<Integer, Button> entry : page1.buttons.entrySet()) {
                Button button = entry.getValue();
                if (button.isAsynchronous()) {
                    ElementOriginLib.inst().getServer().getScheduler().runTaskAsynchronously(ElementOriginLib.inst(), ()->{
                        if (button instanceof ItemButton itemButton) {
                            if (itemButton.getItem() != null) {
                                inventory.setItem(entry.getKey(), itemButton.getItem());
                                return;
                            }
                        }
                        inventory.setItem(entry.getKey(), button.getItemStack());
                    });
                } else {
                    if (button instanceof ItemButton itemButton) {
                        if (itemButton.getItem() != null) {
                            inventory.setItem(entry.getKey(), itemButton.getItem());
                            continue;
                        }
                    }
                    inventory.setItem(entry.getKey(), button.getItemStack());
                }
            }
        }
    }

    @Override
    public void init() {
    }

    @Override
    public void handler(Player player, int slot, InventoryClickEvent event) {
        if (pages.containsKey(page)) {
            Page page1 = getPage(page);
            Button button = page1.getButton(slot);
            if (button == null) {
                if (slot < inventory.getSize()) {
                    event.setCancelled(true);
                } else {
                    if (event.getAction().equals(InventoryAction.MOVE_TO_OTHER_INVENTORY)) {
                        event.setCancelled(true);
                    }
                }
                return;
            }
            event.setCancelled(true);
            clickButton(event, slot, button);
            if (button.isPlaySound()) {
                player.playSound(player.getLocation(), Sound.UI_BUTTON_CLICK, 1.0f, 1.0f);
            }
            InventoryAction action = event.getAction();
            ClickType clickType = ClickType.LEFT_CLICK;
            if (action.equals(InventoryAction.PICKUP_ALL)) {
                clickType = ClickType.LEFT_CLICK;
            } else if (action.equals(InventoryAction.PICKUP_HALF)) {
                clickType = ClickType.RIGHT_CLICK;
            } else if (action.equals(InventoryAction.MOVE_TO_OTHER_INVENTORY)) {
                clickType = ClickType.SHIFT_CLICK;
            }
            if (button.getClickConsumer() != null) {
                button.getClickConsumer().accept(player, clickType);
            }
        }
    }

    public Page getPage(int page) {
        if (page <= 0 || page > maxPage) {
            return null;
        }
        if (!pages.containsKey(page)) {
            pages.put(page, new Page());
        }
        return pages.get(page);
    }

    public void setPage(int page) {
        this.page = page;
    }

    public void next() {
        if (page >= maxPage) {
            return;
        }
        this.page ++;
        initButton();
    }

    public void up() {
        if (page <= 1) {
            return;
        }
        this.page --;
        initButton();
    }

    public int getPage() {
        return page;
    }

    public int getMaxPage() {
        return maxPage;
    }

    public void setMaxPage(int maxPage) {
        this.maxPage = maxPage;
    }

    public void addButton(int page, int index, Button button) {
        Page page1 = pages.getOrDefault(page, new Page());
        page1.buttons.put(index, button);
        pages.put(page, page1);
    }

    public static class Page {

        private Map<Integer, Button> buttons = new HashMap<>();

        public Page() {
        }

        public void addButton(int index, Button button) {
            buttons.put(index, button);
        }

        public Button getButton(int index) {
            return buttons.getOrDefault(index, null);
        }

        public Map<Integer, Button> getButtons() {
            return buttons;
        }

    }

}
