package com.io.yutian.elementoriginlib.item;

import com.io.yutian.elementoriginlib.item.stat.ItemStat;
import com.io.yutian.elementoriginlib.item.stat.StatData;
import com.io.yutian.elementoriginlib.nbt.NBTItem;
import org.bukkit.inventory.ItemStack;
import org.jetbrains.annotations.NotNull;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

public class OriginItem {

    private ItemStack itemStack;
    private NBTItem nbtItem;
    private Map<ItemStat, StatData> stats = new HashMap<>();

    public OriginItem(ItemStack itemStack) {
        this.itemStack = itemStack;
        this.nbtItem = new NBTItem(itemStack);
    }

    public void setStatData(@NotNull ItemStat stat, @NotNull StatData data) {
        this.stats.put(stat, data);
    }

    public void removeStatData(@NotNull ItemStat stat) {
        this.stats.remove(stat);
    }

    public <S extends StatData> S getStatData(@NotNull ItemStat<S> stat) {
        if (!hasStatData(stat)) {
            stat.load(this);
        }
        return (S) this.stats.get(stat);
    }

    public boolean hasStatData(@NotNull ItemStat stat) {
        if (!this.stats.containsKey(stat)) {
            try {
                stat.load(this);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return this.stats.containsKey(stat);
    }

    @NotNull
    public OriginItemStackBuilder builder() {
        return new OriginItemStackBuilder(this);
    }

    @NotNull
    public Set<ItemStat> getStats() {
        return this.stats.keySet();
    }

    public ItemStack getItemStack() {
        return itemStack;
    }

    public NBTItem getNBTItem() {
        return nbtItem;
    }

    @Override
    public OriginItem clone() {
        OriginItem originItem = new OriginItem(itemStack.clone());
        Iterator iterator = this.stats.keySet().iterator();
        while(iterator.hasNext()) {
            ItemStat itemStat = (ItemStat) iterator.next();
            originItem.stats.put(itemStat, this.stats.get(itemStat));
        }
        return originItem;
    }

}
