package com.io.yutian.elementoriginlib.item;

import com.io.yutian.elementoriginlib.item.stat.StatData;
import com.io.yutian.elementoriginlib.nbt.NBTItem;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.ItemMeta;
import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.List;

public class OriginItemStackBuilder {

    private final OriginItem originItem;

    protected ItemStack itemStack;
    protected ItemMeta itemMeta;

    protected NBTItem nbtItem;

    public OriginItemStackBuilder(OriginItem originItem) {
        this.originItem = originItem;
        this.itemStack = originItem.getItemStack();
        this.itemMeta = itemStack.getItemMeta();
        this.nbtItem = originItem.getNBTItem();
    }

    @NotNull
    public ItemStack build() {
        buildNBT();
        return itemStack;
    }

    @NotNull
    public ItemMeta getItemMeta() {
        return itemMeta;
    }

    private void buildNBT() {
        originItem.getStats().forEach(stat -> {
            StatData statData = originItem.getStatData(stat);
            stat.whenApplied(this, statData);
        });
        itemStack = nbtItem.getItemStack();
        itemMeta = itemStack.getItemMeta();
        originItem.getStats().forEach(stat -> {
            StatData statData = originItem.getStatData(stat);
            stat.applyMeta(this, statData);
        });
        itemStack.setItemMeta(itemMeta);
        List<String> lores = itemMeta.hasLore() ? itemMeta.getLore() : new ArrayList<>();
        originItem.getStats().forEach(stat -> {
            StatData statData = originItem.getStatData(stat);
            stat.whenApplyLore(this, statData, lores);
        });
        itemStack.setLore(lores);
    }

    @NotNull
    public NBTItem getNBTItem() {
        return nbtItem;
    }

    public OriginItem getOriginItem() {
        return originItem;
    }

}
