package com.io.yutian.elementoriginlib.item.stat;

import com.io.yutian.elementoriginlib.item.OriginItem;
import com.io.yutian.elementoriginlib.item.OriginItemStackBuilder;
import com.io.yutian.elementoriginlib.nbt.NBTItem;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.List;

public abstract class ItemStat<S extends StatData> {

    @NotNull
    private final String id;
    @NotNull
    private final String nbtPath;

    public ItemStat(@NotNull String id, @NotNull String path) {
        this.id = id;
        this.nbtPath = path;
    }

    public abstract void whenApplied(@NotNull OriginItemStackBuilder itemStackBuilder, @NotNull S statData);

    public void applyMeta(@NotNull OriginItemStackBuilder itemStackBuilder, @NotNull S statData) {
    }

    public void whenApplyLore(@NotNull OriginItemStackBuilder itemStackBuilder, @NotNull S statData, @NotNull List<String> lores) {
    }

    @Nullable
    public abstract S getLoadedNBT(@NotNull NBTItem nbtItem);

    public void load(@NotNull OriginItem originItem) {
        S loadedNBT = getLoadedNBT(originItem.getNBTItem());
        if (loadedNBT != null) {
            originItem.setStatData(this, loadedNBT);
        }
    }

    @NotNull
    public String getId() {
        return id;
    }

    @NotNull
    public String getNBTPath() {
        return nbtPath;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        ItemStat<?> itemStat = (ItemStat<?>) o;
        return id.equals(itemStat.id);
    }

    @Override
    public int hashCode() {
        return id.hashCode();
    }
}
