package com.io.yutian.elementoriginlib.item.stat.type;

import com.io.yutian.elementoriginlib.item.OriginItemStackBuilder;
import com.io.yutian.elementoriginlib.item.stat.ItemStat;
import com.io.yutian.elementoriginlib.item.stat.data.StringListData;
import com.io.yutian.elementoriginlib.nbt.NBTItem;
import com.io.yutian.elementoriginlib.nbt.NBTList;
import com.io.yutian.elementoriginlib.nbt.NBTString;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.ArrayList;
import java.util.List;

public abstract class StringListStat extends ItemStat<StringListData> {

    public StringListStat(@NotNull String id, @NotNull String path) {
        super(id, path);
    }

    @Override
    public void whenApplied(@NotNull OriginItemStackBuilder itemStackBuilder, @NotNull StringListData statData) {
        NBTItem nbtItem = itemStackBuilder.getNBTItem();
        NBTList<NBTString> nbtList = new NBTList<>();
        statData.getList().forEach(s -> nbtList.add(new NBTString(s)));
        nbtItem.editTag(nbtCompound -> nbtCompound.put(getNBTPath(), nbtList));
    }

    @Nullable
    @Override
    public StringListData getLoadedNBT(@NotNull NBTItem nbtItem) {
        if (!nbtItem.has(getNBTPath(), NBTList.TYPE_ID)) {
            return new StringListData(new ArrayList<>());
        }
        NBTList<NBTString> nbtList = (NBTList<NBTString>) nbtItem.get(getNBTPath());
        List<String> list = new ArrayList<>();
        for (int i = 0; i < nbtList.size(); i++) {
            list.add(nbtList.get(i).getString());
        }
        return new StringListData(list);
    }

}
