package com.io.yutian.elementoriginlib.lang;

import org.bukkit.ChatColor;
import org.bukkit.configuration.file.FileConfiguration;
import org.bukkit.configuration.file.YamlConfiguration;
import org.bukkit.plugin.Plugin;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class Lang {

    private static Map<Plugin, List<File>> langFileMap = new HashMap<>();

    private static Map<String, String> langMap = new HashMap<>();

    public static String get(String key) {
        return langMap.getOrDefault(key, "§o"+key);
    }

    public static String get(String key, Object... args) {
        String s = langMap.getOrDefault(key, "§o"+key);
        for (int i = 0; i < args.length; i++) {
            s = s.replace("$"+i, String.valueOf(args[i]));
        }
        return s;
    }

    public static void reload(Plugin plugin) {
        if (!langFileMap.containsKey(plugin)) {
            return;
        }
        List<File> list = langFileMap.get(plugin);
        list.forEach(Lang::loadFile);
    }

    public static void reload() {
        langMap.clear();
        for (List<File> files : langFileMap.values()) {
            files.forEach(Lang::loadFile);
        }
    }

    private static void loadFile(File file) {
        FileConfiguration fileConfiguration = YamlConfiguration.loadConfiguration(file);
        for (String key : fileConfiguration.getKeys(true)) {
            if (!fileConfiguration.isString(key)) {
                continue;
            }
            String string = fileConfiguration.getString(key);
            string = ChatColor.translateAlternateColorCodes('&', string);
            langMap.put(key, string);
        }
    }

    public static void registerLangFile(Plugin plugin) {
        registerLangFile(plugin, getFile(plugin));
    }

    public static void registerLangFile(Plugin plugin, File file) {
        if (!file.exists()) {
            return;
        }
        List<File> files = langFileMap.getOrDefault(plugin, new ArrayList<>());
        files.add(file);
        langFileMap.put(plugin, files);
        loadFile(file);
    }

    public static File getFile(Plugin plugin) {
        File file = new File(plugin.getDataFolder()+File.separator+ "lang.yml");
        if (!file.exists()) {
            plugin.saveResource("lang.yml", false);
        }
        return file;
    }

}
