package com.io.yutian.elementoriginlib.nbt;

import net.minecraft.nbt.*;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public interface INBT {

    Object getValue();

    byte getTypeId();

    private static String getNBTTag(byte typeByte) {
        return switch (typeByte) {
            case 1 -> "NBTTagByte";
            case 2 -> "NBTTagShort";
            case 3 -> "NBTTagInt";
            case 4 -> "NBTTagLong";
            case 5 -> "NBTTagFloat";
            case 6 -> "NBTTagDouble";
            case 7 -> "NBTTagByteArray";
            case 8 -> "NBTTagString";
            case 9 -> "NBTTagList";
            case 10 -> "NBTTagCompound";
            case 11 -> "NBTTagIntArray";
            case 12 -> "NBTTagLongArray";
            default -> null;
        };
    }

    static INBT asObject(Object object) {
        if (object instanceof Byte b) {
            return new NBTByte(b);
        } else if (object instanceof Short value) {
            return new NBTShort(value);
        } else if (object instanceof Integer value) {
            return new NBTInt(value);
        } else if (object instanceof Long value) {
            return new NBTLong(value);
        } else if (object instanceof Float value) {
            return new NBTFloat(value);
        } else if (object instanceof Double value) {
            return new NBTDouble(value);
        } else if (object instanceof byte[] value) {
            return new NBTByteArray(value);
        } else if (object instanceof String value) {
            return new NBTString(value);
        } else if (object instanceof int[] value) {
            return new NBTIntArray(value);
        } else if (object instanceof long[] value) {
            return new NBTLongArray(value);
        } else if (object instanceof Boolean b) {
            return new NBTByte((byte) (b ? 1 : 0));
        }
        return null;
    }

    static net.minecraft.nbt.NBTBase asNMS(INBT nbt) {
        byte type = nbt.getTypeId();
        switch (type) {
            case 1 -> {
                NBTByte nbtByte = (NBTByte) nbt;
                return NBTTagByte.a(nbtByte.getByte());
            }
            case 2 -> {
                NBTShort nbtShort = (NBTShort) nbt;
                return NBTTagShort.a(nbtShort.getShort());
            }
            case 3 -> {
                NBTInt nbtInt = (NBTInt) nbt;
                return NBTTagInt.a(nbtInt.getInt());
            }
            case 4 -> {
                NBTLong nbtLong = (NBTLong) nbt;
                return NBTTagLong.a(nbtLong.getLong());
            }
            case 5 -> {
                NBTFloat nbtfloat = (NBTFloat) nbt;
                return NBTTagFloat.a(nbtfloat.getFloat());
            }
            case 6 -> {
                NBTDouble nbtDouble = (NBTDouble) nbt;
                return NBTTagDouble.a(nbtDouble.getDouble());
            }
            case 7 -> {
                NBTByteArray nbtByteArray = (NBTByteArray) nbt;
                return new NBTTagByteArray(nbtByteArray.getByteArray());
            }
            case 8 -> {
                NBTString nbtString = (NBTString) nbt;
                return NBTTagString.a(nbtString.getString());
            }
            case 9 -> {
                NBTList nbtTagList = (NBTList) nbt;
                List<net.minecraft.nbt.NBTBase> list = new ArrayList<>();
                for (Object base : nbtTagList.getList()) {
                    list.add(asNMS((INBT) base));
                }
                NBTTagList nbtTagList1 = new NBTTagList();
                for (net.minecraft.nbt.NBTBase nbt1 : list) {
                    nbtTagList1.add(nbt1);
                }
                return nbtTagList1;
            }
            case 10 -> {
                NBTCompound nbtCompound = (NBTCompound) nbt;
                NBTTagCompound nbtTagCompound = new NBTTagCompound();
                for (String key : nbtCompound.keySet()) {
                    INBT nbt1 = nbtCompound.get(key);
                    nbtTagCompound.a(key, asNMS(nbt1));
                }
                return nbtTagCompound;
            }
            case 11 -> {
                NBTIntArray nbtIntArray = (NBTIntArray) nbt;
                return new NBTTagIntArray(nbtIntArray.getIntArray());
            }
            case 12 -> {
                NBTLongArray nbtLongArray = (NBTLongArray) nbt;
                return new NBTTagLongArray(nbtLongArray.getLongArray());
            }
        }
        return null;
    }

    static INBT as(net.minecraft.nbt.NBTBase nbtBase) {
        byte type = nbtBase.b();
        switch (type) {
            case 1:
                NBTTagByte nbtTagByte = (NBTTagByte) nbtBase;
                return new NBTByte(nbtTagByte.i());
            case 2:
                NBTTagShort nbtTagShort = (NBTTagShort) nbtBase;
                return new NBTShort(nbtTagShort.h());
            case 3:
                NBTTagInt nbtTagInt = (NBTTagInt) nbtBase;
                return new NBTInt(nbtTagInt.g());
            case 4:
                NBTTagLong nbtTagLong = (NBTTagLong) nbtBase;
                return new NBTLong(nbtTagLong.f());
            case 5:
                NBTTagFloat nbtTagFloat = (NBTTagFloat) nbtBase;
                return new NBTFloat(nbtTagFloat.k());
            case 6:
                NBTTagDouble nbtTagDouble = (NBTTagDouble) nbtBase;
                return new NBTDouble(nbtTagDouble.j());
            case 7:
                NBTTagByteArray tagByteArray = (NBTTagByteArray) nbtBase;
                return new NBTByteArray(tagByteArray.e());
            case 8:
                NBTTagString nbtTagString = (NBTTagString) nbtBase;
                return new NBTString(nbtTagString.m_());
            case 9:
                NBTTagList nbtTagList = (NBTTagList) nbtBase;
                List<INBT> list = new ArrayList<>();
                for (net.minecraft.nbt.NBTBase base : nbtTagList) {
                    list.add(as(base));
                }
                return new NBTList(list);
            case 10:
                NBTTagCompound nbtTagCompound = (NBTTagCompound) nbtBase;
                Map<String, INBT> map = new HashMap<>();
                for (String key : nbtTagCompound.e()) {
                    map.put(key, as(nbtTagCompound.c(key)));
                }
                return new NBTCompound(map);
            case 11:
                NBTTagIntArray nbtTagIntArray = (NBTTagIntArray) nbtBase;
                return new NBTIntArray(nbtTagIntArray.g());
            case 12:
                NBTTagLongArray nbtTagLongArray = (NBTTagLongArray) nbtBase;
                return new NBTLongArray(nbtTagLongArray.g());
        }
        return null;
    }

}
