package com.io.yutian.elementoriginlib.nbt;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

public class NBTList<T extends INBT> implements INBT {

    public static byte TYPE_ID = 9;

    private List<T> value = new ArrayList<>();
    private byte type = 0;

    public NBTList(List<T> value) {
        this.value = value;
    }

    public NBTList() {
    }

    public List<T> getList() {
        return value;
    }

    public int size() {
        return value.size();
    }

    public T get(int index) {
        if (index > value.size()) {
            return null;
        }
        return value.get(index);
    }

    public void add(T nbtBase) {
        if (nbtBase.getTypeId() == 0) {
            return;
        }
        if (this.type == 0) {
            this.type = nbtBase.getTypeId();
        } else if (this.type != nbtBase.getTypeId()) {
            return;
        }
        this.value.add(nbtBase);
    }

    public void remove(int index) {
        this.value.remove(index);
    }

    public boolean isEmpty() {
        return value.isEmpty();
    }

    @Override
    public Object getValue() {
        return this.value;
    }

    @Override
    public byte getTypeId() {
        return TYPE_ID;
    }

    public byte getType() {
        return type;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        NBTList<?> nbtList = (NBTList<?>) o;
        return type == nbtList.type && Objects.equals(value, nbtList.value);
    }

    @Override
    public int hashCode() {
        int result = Objects.hashCode(value);
        result = 31 * result + type;
        return result;
    }

    @Override
    public String toString() {
        return "NBTList{" +
                "value=" + value +
                ", type=" + type +
                '}';
    }

}
