package com.io.yutian.elementoriginlib.serialize.serializers;

import com.io.yutian.elementoriginlib.serialize.Serializer;
import com.mojang.brigadier.exceptions.CommandSyntaxException;
import net.minecraft.nbt.*;
import org.bukkit.craftbukkit.v1_20_R1.inventory.CraftItemStack;
import org.bukkit.inventory.ItemStack;
import org.json.JSONArray;
import org.json.JSONObject;

public class ItemStackSerializer implements Serializer<ItemStack> {

    @Override
    public Object serialize(ItemStack value) {
        JSONObject jsonObject = new JSONObject();
        net.minecraft.world.item.ItemStack nmsItemStack = CraftItemStack.asNMSCopy(value);
        jsonObject.put("id", nmsItemStack.c().d().j().g());
        jsonObject.put("Count", nmsItemStack.J());
        NBTTagCompound nbtTagCompound = nmsItemStack.w();
        JSONObject tagJsonObject = new JSONObject();
        for (String key : nbtTagCompound.e()) {
            NBTBase nbtBase = nbtTagCompound.c(key);
            tagJsonObject.put(key, serializeNBT(nbtBase));
        }
        jsonObject.put("tag", tagJsonObject);
        return jsonObject;
    }

    private Object serializeNBT(NBTBase nbtBase) {
        byte type = nbtBase.b();
        switch (type) {
            case 1:
                NBTTagByte nbtTagByte = (NBTTagByte) nbtBase;
                return nbtTagByte.i();
            case 2:
                NBTTagShort nbtTagShort = (NBTTagShort) nbtBase;
                return nbtTagShort.h();
            case 3:
                NBTTagInt nbtTagInt = (NBTTagInt) nbtBase;
                return nbtTagInt.g();
            case 4:
                NBTTagLong nbtTagLong = (NBTTagLong) nbtBase;
                return nbtTagLong.f();
            case 5:
                NBTTagFloat nbtTagFloat = (NBTTagFloat) nbtBase;
                return nbtTagFloat.k();
            case 6:
                NBTTagDouble nbtTagDouble = (NBTTagDouble) nbtBase;
                return nbtTagDouble.j();
            case 7:
                NBTTagByteArray tagByteArray = (NBTTagByteArray) nbtBase;
                return tagByteArray.e();
            case 8:
                NBTTagString nbtTagString = (NBTTagString) nbtBase;
                return nbtTagString.m_();
            case 9:
                NBTTagList nbtTagList = (NBTTagList) nbtBase;
                JSONArray jsonArray = new JSONArray();
                for (net.minecraft.nbt.NBTBase base : nbtTagList) {
                    jsonArray.put(serializeNBT(base));
                }
                return jsonArray;
            case 10:
                NBTTagCompound nbtTagCompound = (NBTTagCompound) nbtBase;
                JSONObject jsonObject = new JSONObject();
                for (String key : nbtTagCompound.e()) {
                    jsonObject.put(key, serializeNBT(nbtTagCompound.c(key)));
                }
                return jsonObject;
            case 11:
                NBTTagIntArray nbtTagIntArray = (NBTTagIntArray) nbtBase;
                return nbtTagIntArray.g();
            case 12:
                NBTTagLongArray nbtTagLongArray = (NBTTagLongArray) nbtBase;
                return nbtTagLongArray.g();
        }
        return null;
    }

    @Override
    public ItemStack deserialize(Object value) {
        JSONObject jsonObject = (JSONObject) value;
        String arg = jsonObject.toString();
        NBTTagCompound nbtTagCompound;
        try {
            nbtTagCompound = net.minecraft.nbt.MojangsonParser.a(arg);
        } catch (CommandSyntaxException e) {
            e.printStackTrace();
            return null;
        }
        net.minecraft.world.item.ItemStack nmsItemStack = net.minecraft.world.item.ItemStack.a(nbtTagCompound);
        return CraftItemStack.asCraftMirror(nmsItemStack);
    }

}
