package com.io.yutian.elementoriginlib.sql.builder.impl;

import com.io.yutian.elementoriginlib.sql.action.PreparedSQLUpdateActionImpl;
import com.io.yutian.elementoriginlib.sql.api.SQLAction;
import com.io.yutian.elementoriginlib.sql.api.action.PreparedSQLUpdateAction;
import com.io.yutian.elementoriginlib.sql.api.builder.DeleteBuilder;
import com.io.yutian.elementoriginlib.sql.manager.SQLManagerImpl;
import org.jetbrains.annotations.NotNull;

import java.util.Objects;

import static com.io.yutian.elementoriginlib.sql.api.SQLBuilder.withBackQuote;

public class DeleteBuilderImpl
        extends AbstractConditionalBuilder<DeleteBuilder, SQLAction<Integer>>
        implements DeleteBuilder {

    protected final String tableName;

    public DeleteBuilderImpl(@NotNull SQLManagerImpl manager, @NotNull String tableName) {
        super(manager);
        Objects.requireNonNull(tableName);
        this.tableName = tableName;
    }

    @Override
    public PreparedSQLUpdateAction<Integer> build() {

        StringBuilder sqlBuilder = new StringBuilder();

        sqlBuilder.append("DELETE FROM ").append(withBackQuote(getTableName()));

        if (hasConditions()) sqlBuilder.append(" ").append(buildConditionSQL());
        if (limit > 0) sqlBuilder.append(" ").append(buildLimitSQL());

        return new PreparedSQLUpdateActionImpl<>(
                getManager(), Integer.class, sqlBuilder.toString(),
                (hasConditionParams() ? getConditionParams() : null)
        );
    }

    @Override
    public String getTableName() {
        return tableName;
    }


    @Override
    protected DeleteBuilderImpl getThis() {
        return this;
    }

}
