package com.io.yutian.elementoriginlib.util;

import com.github.luben.zstd.Zstd;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.compress.archivers.ArchiveEntry;
import org.apache.commons.compress.archivers.ArchiveInputStream;
import org.apache.commons.compress.archivers.ArchiveOutputStream;
import org.apache.commons.compress.archivers.zip.ZipArchiveEntry;
import org.apache.commons.compress.archivers.zip.ZipArchiveInputStream;
import org.apache.commons.compress.archivers.zip.ZipArchiveOutputStream;
import org.apache.commons.compress.utils.IOUtils;

import java.io.*;
import java.nio.file.*;
import java.nio.file.attribute.BasicFileAttributes;
import java.util.Objects;

public class CompressUtils {

    public static String zstdString(String unzip) {
        byte[] compress = Zstd.compress(unzip.getBytes());
        return Base64.encodeBase64String(compress);
    }
    public static String unZstdString(String zip) {
        byte[] decode = Base64.decodeBase64(zip);
        Long size = Zstd.decompressedSize(decode);
        byte[] decompress = Zstd.decompress(decode, size.intValue());
        return new String(decompress);
    }

    public static void zip(File file, File file1) {
        try {
            OutputStream outputStream = new FileOutputStream(file1);
            zip(file, outputStream);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static void zip(File file, OutputStream outputStream) throws IOException {
        try (BufferedOutputStream bufferedOutputStream = new BufferedOutputStream(outputStream);
             ArchiveOutputStream out = new ZipArchiveOutputStream(bufferedOutputStream)) {
            Path start = Paths.get(file.toURI());
            Files.walkFileTree(start, new SimpleFileVisitor<>() {

                @Override
                public FileVisitResult preVisitDirectory(Path dir, BasicFileAttributes attrs) throws IOException {
                    ArchiveEntry entry = new ZipArchiveEntry(dir.toFile(), start.relativize(dir).toString());
                    out.putArchiveEntry(entry);
                    out.closeArchiveEntry();
                    return super.preVisitDirectory(dir, attrs);
                }

                @Override
                public FileVisitResult visitFile(Path file, BasicFileAttributes attrs) throws IOException {
                    try (InputStream input = new FileInputStream(file.toFile())) {
                        ArchiveEntry entry = new ZipArchiveEntry(file.toFile(), start.relativize(file).toString());
                        out.putArchiveEntry(entry);
                        IOUtils.copy(input, out);
                        out.closeArchiveEntry();
                    }
                    return super.visitFile(file, attrs);
                }

            });

        }
    }

    public static void unzip(File file, String destDir) {
        try {
            try (InputStream inputStream = new FileInputStream(file);) {
                unzip(inputStream, destDir);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    public static void unzip(InputStream inputStream, String destDir) {
        try {
            try (BufferedInputStream bufferedInputStream = new BufferedInputStream(inputStream); ArchiveInputStream in = new ZipArchiveInputStream(bufferedInputStream)) {
                ArchiveEntry entry;
                while (Objects.nonNull(entry = in.getNextEntry())) {
                    File file = Paths.get(destDir, entry.getName()).toFile();
                    if (in.canReadEntryData(entry)) {
                        if (entry.isDirectory()) {
                            if (!file.exists()) {
                                file.mkdirs();
                            }
                        } else {
                            try (OutputStream out = new FileOutputStream(file)) {
                                IOUtils.copy(in, out);
                            }
                        }
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

}