package com.io.yutian.elementoriginlib.util;

import org.bukkit.plugin.Plugin;

import java.io.*;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class FileUtil {

    public static List<File> getFilesForFolder(File folder) {
        List<File> files = new ArrayList<>();
        if (!folder.isDirectory()) {
            throw new IllegalArgumentException("File \"" + folder.getName() + "\" is not a directory");
        }
        File[] listFiles = folder.listFiles();
        for (File file : listFiles) {
            if (file.isDirectory()) {
                files.addAll(getFilesForFolder(file));
            } else {
                files.add(file);
            }
        }
        return files;
    }

    public static List<File> getSubFolders(File folder) {
        List<File> subFolders = new ArrayList<>();
        if (!folder.isDirectory()) {
            throw new IllegalArgumentException("File \"" + folder.getName() + "\" is not a directory");
        }
        File[] listFiles = folder.listFiles();
        for (File file : listFiles) {
            if (file.isDirectory()) {
                subFolders.add(file);
            }
        }
        return subFolders;
    }

    public static void copy(File inFile, File outFile) {
        if (inFile.isDirectory()) {
            copyDir(inFile, outFile, new String[0]);
        } else {
            copyFile(inFile, outFile);
        }
    }

    public static boolean copyFile(File inFile, File outFile) {
        return org.bukkit.util.FileUtil.copy(inFile, outFile);
    }

    public static void copyDir(File inDir, File outDir, String... exclude) {
        if (!outDir.exists()) {
            outDir.mkdir();
        }
        String[] list = inDir.list();
        for (String p : list) {
            if (!Arrays.asList(exclude).contains(p)) {
                copy(new File(inDir, p), new File(outDir, p));
            }
        }
    }

    public static boolean removeDir(File dir) {
        if (dir.isDirectory()) {
            for (File f : dir.listFiles()) {
                if (!removeDir(f)) {
                    return false;
                }
            }
        }
        return dir.delete();
    }

    public static FileOutputStream getOutputStream(File file) {
        try {
            return new FileOutputStream(file);
        } catch (Exception e) {
            return null;
        }
    }

    public static FileInputStream getInputStream(File file) {
        try {
            return new FileInputStream(file);
        } catch (Exception e) {
            return null;
        }
    }

    public static File getFile(Plugin plugin, String path, String fileName) {
        File p = (path == null || path.equals("")) ? plugin.getDataFolder() : new File(plugin.getDataFolder() + File.separator + path);
        if (!p.exists()) {
            p.mkdirs();
        }
        File file = new File(p + File.separator + fileName);
        return file;
    }

}