package com.io.yutian.elementoriginlib.util;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Random;

public class RandomBuilder<T> {

    private Map<T, Double> valueMap;

    private List<Element> elementList = new ArrayList<>();
    private double maxElement = 0;

    public RandomBuilder(Map<T, Double> map) {
        valueMap = map;

        double minElement = 0;

        for (Map.Entry<T, Double> entry : map.entrySet()) {
            minElement = maxElement;
            maxElement += entry.getValue();
            elementList.add(new Element(entry.getKey(), minElement, maxElement));
        }
    }

    public T random() {
        return random(new Random());
    }

    public T random(Random random) {
        double d = random.nextDouble() * maxElement;
        if (d == 0) {
            d = 0.1 * maxElement;
        }
        int size = valueMap.size();
        for (int i = 0; i < elementList.size(); i++) {
            Element element = elementList.get(i);
            if (element.isContainKey(d)) {
                return element.getValue();
            }
        }
        return null;
    }

    public double getMaxElement() {
        return maxElement;
    }

    public List<Element> getElementList() {
        return elementList;
    }

    public Map<T, Double> getValueMap() {
        return valueMap;
    }

    private class Element {

        private T value;

        private final double minElement;
        private final double maxElement;

        public Element(T value, double element0, double element1) {
            this.value = value;
            this.minElement = Math.min(element0, element1);
            this.maxElement = Math.max(element0, element1);
        }

        public T getValue() {
            return value;
        }

        public boolean isContainKey(double element) {
            return element > minElement && element <= maxElement;
        }

    }



}
